# coding: utf-8

from __future__ import absolute_import
from __future__ import unicode_literals

from abc import ABCMeta
from abc import abstractmethod

import six


class BaseBackend(six.with_metaclass(ABCMeta)):

    """
    Базовый класс для всех бэкендов.    
    Бэкенд - сущность определяющая логику обработки запросов на вход и выход,
    
    В качестве бэкенда можно рассматривать аутентификацию по логину и паролю
    осуществляющаяся средствами django, а также аутентификацию через удаленные
    сервера, такие как: ЕСИА, ВКонтакте, Яндекс, Google и т.д.
    
    :param name: Наименование бэкенда.
    :type name: str or unicode
    """

    def __init__(self, name):
        self.name = name

    @abstractmethod
    def get_login_url(self, request):
        """Метод должен возвращать URL адрес, на который необходимо 
        отправлять запрос на вход.

        :param request: Запрос
        :type request: django.http.request.HttpRequest
        :return: URL адрес.
        :rtype: str
        """

    @abstractmethod
    def process_login(self, request):
        """Метод должен обрабатывать запрос на вход и возвращать объект
        пользователя.

        :param request: Запрос
        :type request: django.http.request.HttpRequest
        :return: Объект класса пользователя Django
        :rtype: django.contrib.auth.models.User
        """

    @abstractmethod
    def get_logout_url(self, request):
        """Метод должен возвращать URL адрес для выхода.

        :param request: Запрос
        :type request: django.http.request.HttpRequest
        :return: URL адрес.
        :rtype: str
        """

    @abstractmethod
    def process_logout(self, request):
        """Метод должен обрабатывать запрос на выход.

        :param request: Запрос
        :type request: django.http.request.HttpRequest
        :return: True или False
        :rtype: bool
        """
