# coding: utf-8

from __future__ import absolute_import
from __future__ import unicode_literals

from abc import ABCMeta, abstractmethod

import six


class BaseProtocol(six.with_metaclass(ABCMeta)):

    """
    Базовый класс для классов реализующих логику работы с конкретными 
    протоколами. Например, OAuth2.0, SMAL2.
    
    :param user_cls: Класс который будет представлять информацию о пользователе
        стороннего сервера авторизации.
    :type user_cls: type
    
    :param config: Объект класса auth.config.Config с настройками конкретного
        протокола.
    :type config: auth.config.Config
    """

    def __init__(self, user_cls, config):
        super(BaseProtocol, self).__init__()
        self.user_cls = user_cls
        self.config = config

    @abstractmethod
    def get_login_url(self, request):
        """Метод должен формировать и возвращать URL адрес для переадресации
        на сервер авторизации для осуществления входа по конкретному протоколу.
        
        :param request: Запрос.
        :type request: django.http.httpRequest
        :return: URL адрес.
        :rtype: str or unicode
        """

    @abstractmethod
    def process_login(self, request):
        """Метод должен обрабатывать запрос на вход полученного с сервера 
        авторизации и возвращать объект класса user_cls с информацией о 
        пользователе.
        
        :param request: Запрос.
        :type request: django.http.httpRequest
        
        :return: Объект класса с информацие о пользователе
        :rtype: auth.backends.base.remote.user.BaseRemoteUser
        """

    @abstractmethod
    def get_logout_url(self, request):
        """Метод должен возвращать URL адрес для переадресации на выход.
        
        :param request: Запрос.
        :type request: django.http.HttpRequest
        :return: URL адресю
        :rtype: str or unicode
        """

    @abstractmethod
    def process_logout(self, request):
        """Метод должен обрабатывает запрос на выход, и возвращать True если 
        запрос корректный, в противном случае выбрасывать исключение AuthError.
        
        :param request: Запрос.
        :type request: django.http.HttpRequest
        :return: True
        :rtype: bool
        """

    @classmethod
    @abstractmethod
    def from_config(cls, config):
        """Метод должен реализовывать создание объекта протокола из настроек
        config и возвращать его.
        
        :param config: Настройки протокола.
        :type config: auth.config.Config
        :return: Объект класса протокола.
        :rtype: type(cls)
        """
