# coding: utf-8

from __future__ import absolute_import
from __future__ import unicode_literals

from collections import Mapping
from os import path

from django.conf import settings
from django.contrib.sessions.middleware import SessionMiddleware
from django.test.client import RequestFactory
from django.test.testcases import TestCase
from oauthlib.oauth2 import WebApplicationClient

from m3_auth.backends.base.remote.user import BaseRemoteUser
from m3_auth.backends.oauth2.api import BaseOAuthServerApi
from m3_auth.backends.oauth2.protocol import OAuthProtocol
from m3_auth.config import Config
from m3_auth.exceptions import AuthError
from tests.testapp.tests.utils import MockRequests


class TestOAuthServerApi(BaseOAuthServerApi):

    """Тестовый класс API."""

    def get_person_info(self, request, client):
        return {
            'user_id': 123,
            'surname': 'Супер',
            'first_name': 'Пользователь',
            'patronymic': 'Системы'
        }


class TestOAuthUser(BaseRemoteUser):

    """Тестовый класс для проверки класса протокола."""


class TestOAuthProtocol(TestCase):

    """Проверка класса протокола OAuthProtocol."""

    def setUp(self):
        super(TestOAuthProtocol, self).setUp()
        self.config = Config({
            "client_id": "id",
            "login": {
                "url": "https://localhost:8000/oauth2/login",
                "redirect_url": "http://localhost:8000/process_login",
                "request_params": ["timestamp", "state", "client_secret"]
            },
            "logout": {
                "url": "https://localhost:8000/process_logout",
                "redirect_url": "some_redirect_url"
            },
            "token": {
                "url": "https://localhost:8000/oauth2/token",
                "redirect_url": "http://localhost:8000/process_login",
                "request_params": ["client_secret", "timestamp", "state"]
            },
            "person_info": {
                "person_url": "http://localhost:8000/oauth2/person",
            },
            "client_secret": "secret",
            "scope": ["openid", "fullname"],
            "ssl_verification": False
        })

        self.config_with_signer = Config(dict(self.config))
        self.config_with_signer.client_secret = Config({
            "signer_cls": "m3_auth.backends.oauth2.signers.PKCS7Signer",
            "params": {
                "certificate_file": path.join(settings.CERTS_PATH, 'sp.crt'),
                "private_key_file": path.join(settings.CERTS_PATH, 'sp.key')
            }
        })

        self.api = TestOAuthServerApi(Config())
        self.protocol = OAuthProtocol(TestOAuthUser, self.api, self.config)
        self.protocol_with_signer = OAuthProtocol(
            TestOAuthUser, self.api, self.config_with_signer)
        self.factory = RequestFactory()

    def create_request(self, *args, **kwargs):
        request = self.factory.post(*args, **kwargs)
        SessionMiddleware().process_request(request)
        return request

    def test_get_login_url(self):
        """Проверка метода get_login_url."""
        request = self.create_request('/')
        login_url = self.protocol.get_login_url(request)
        request = self.create_request(login_url)

        for param, value in [
            ('timestamp', None),
            ('state', self.protocol._get_state(request)),
            ('redirect_uri', self.config.login.redirect_url),
            ('response_type', 'code'),
            ('client_id', self.config.client_id),
            ('client_secret', self.config.client_secret),
            ('scope', ' '.join(self.config.scope))
        ]:
            self.assertIn(param, request.GET)
            if value:
                self.assertEqual(request.GET[param], value)

        # Проверка фильтрации параметров
        request = self.create_request('/')
        self.config.login.request_params = ['state']
        login_url = self.protocol.get_login_url(request)
        request = self.factory.post(login_url)

        self.assertIn('state', request.GET)
        self.assertNotIn('client_secret', request.GET)
        self.assertNotIn('timestamp', request.GET)

    def test_process_login(self):
        """Проверка метода process_login."""

        # Проверка запроса с ошибкой
        request = self.create_request('/', {
            'error': True, 'error_description': 'some_description'})
        self.assertRaises(AuthError, self.protocol.process_login, request)

        # Проверка корректного запроса
        request = self.create_request('/', {'code': 'some_code'})
        with MockRequests(self.client):
            result = self.protocol.process_login(request)

        self.assertIsInstance(result, TestOAuthUser)
        token = self.protocol._get_oauth_token(request)
        self.assertEqual(token['access_token'], 'ACCESS_TOKEN')

    def test_get_logout_url(self):
        """Проверка метода get_logout_url."""
        request = self.create_request('/')
        logout_url = self.protocol.get_logout_url(request)
        self.assertEqual(logout_url, self.config.logout.url)

        # С параметрами
        self.config.logout.request_params = ['redirect_url', 'client_id']
        logout_url = self.protocol.get_logout_url(request)
        request = self.factory.post(logout_url)

        for param, value in [
            ('redirect_url', self.config.logout.redirect_url),
            ('client_id', self.config.client_id)
        ]:
            self.assertIn(param, request.GET)
            self.assertEqual(request.GET[param], value)

    def test_process_logout(self):
        """Проверка метода process_logout."""
        request = self.create_request('/')
        self.assertTrue(self.protocol.process_logout(request))

    def test_refresh_token(self):
        """Проверка метода refresh_token."""
        request = self.create_request('/', {'code': 'code'})

        with MockRequests(self.client):
            token = self.protocol.get_token(request)

        # Проверка вызова без параметра refresh_token
        refresh_token = token.pop('refresh_token')
        result = self.protocol.refresh_token(request)
        token['refresh_token'] = refresh_token

        self.assertIsNone(result)

        # Проверка ответа с ошибкой
        token['refresh_token'] = 'error'
        with MockRequests(self.client):
            self.assertRaises(AuthError, self.protocol.refresh_token, request)
        token['refresh_token'] = refresh_token

        # Проверка корректного запроса
        with MockRequests(self.client):
            self.protocol.refresh_token(request)

        refreshed_token = self.protocol._get_oauth_token(request)

        data = refreshed_token['post_params']
        self.assertEqual(data['grant_type'], 'refresh_token')
        self.assertEqual(data['redirect_uri'], self.config.token.redirect_url)
        self.assertEqual(data['client_id'], self.config.client_id)
        self.assertEqual(data['client_secret'], self.config.client_secret)

        # Проверка корректного запроса с фильтрацией параметров
        with MockRequests(self.client):
            self.config.token.request_params = []
            self.protocol.refresh_token(request)

        refreshed_token = self.protocol._get_oauth_token(request)
        self.assertNotIn('client_secret', refreshed_token['post_params'])
        self.assertNotIn('timestamp', refreshed_token['post_params'])
        self.assertNotIn('state', refreshed_token['post_params'])

    def test_get_token(self):
        """Проверка метода get_token."""

        # Проверка корректного запроса
        request = self.create_request('/', {'code': 'some_code'})
        with MockRequests(self.client):
            token = self.protocol.get_token(request)
            self.assertTrue(token)
            self.assertIsInstance(token, Mapping)

        # Проверка переданных параметров
        data = token['post_params']
        self.assertEqual(data['code'], 'some_code')
        self.assertEqual(data['redirect_uri'], self.config.token.redirect_url)
        self.assertEqual(data['client_id'], self.config.client_id)
        self.assertEqual(data['client_secret'], self.config.client_secret)
        self.assertEqual(data['grant_type'], 'authorization_code')

        # Проверка ответа с ошибкой
        request = self.create_request('/', {'code': 'error'})
        with MockRequests(self.client):
            self.assertRaises(AuthError, self.protocol.get_token, request)

        # Проверка переданных параметров с фильтрацией
        request = self.create_request('/', {'code': 'some_code'})
        with MockRequests(self.client):
            self.config.token.request_params = []
            token = self.protocol.get_token(request)

        self.assertNotIn('client_secret', token['post_params'])
        self.assertNotIn('timestamp', token['post_params'])
        self.assertNotIn('state', token['post_params'])

    def test_get_client(self):
        """Проверка метода _get_client."""
        client = self.protocol._get_client()
        self.assertIsInstance(client, WebApplicationClient)

        client = self.protocol._get_client({'access_token': 'some_token'})
        self.assertEqual(client.access_token, 'some_token')

    def test_get_client_secret(self):
        """Проверка метода _get_client_secret."""
        request = self.create_request('/')
        self.assertTrue(self.protocol._get_client_secret({}))
        self.assertTrue(self.protocol_with_signer._get_client_secret({
            'client_id': self.config_with_signer.client_id,
            'scope': ' '.join(self.config_with_signer.scope),
            'timestamp': self.protocol_with_signer._get_timestamp(),
            'state': self.protocol_with_signer._set_state(request)
        }))

    def test_get_request_params(self):
        """Проверка метода _get_request_params."""
        request = self.create_request('/')
        state, timestamp, secret = self.protocol._get_request_params(request)
        self.assertTrue(state)
        self.assertTrue(secret)
        self.assertTrue(timestamp)

    def test_set_and_get_state(self):
        """Проверка методов _set_state и _get_state"""
        request = self.create_request('/')

        self.protocol._set_state(request)
        self.assertTrue(self.protocol._get_state(request))

        self.protocol._set_state(request, 'some_state')
        self.assertEqual(self.protocol._get_state(request), 'some_state')

    def test_get_headers(self):
        """Проверка метода _get_headers."""
        self.assertTrue(self.protocol._get_headers())

    def test_save_and_get_oauth_token(self):
        """Проверка методов _save_oauth_token и _get_oauth_token."""
        token = {'access_token': 'access', 'refresh_token': '...'}
        request = self.create_request('/')

        self.protocol._save_oauth_token(request, token)
        self.assertEqual(self.protocol._get_oauth_token(request), token)

    def test_from_config(self):
        """Проверка метода from_config."""
        config = Config({
            'user_cls': '.'.join((__name__, TestOAuthUser.__name__)),
            'config': {},
            'api_cls': '.'.join((__name__, TestOAuthServerApi.__name__)),
            'api_config': {}
        })

        protocol = OAuthProtocol.from_config(config)
        self.assertIsInstance(protocol, OAuthProtocol)
        self.assertIs(protocol.user_cls, TestOAuthUser)
        self.assertIs(protocol.config, config.config)
